@@ -2,8 +2,9 @@
 
 use anchor_lang::prelude::*;
 use anchor_lang::Key;
+use vipers::assert_keys_eq;
+use vipers::invariant;
 use vipers::validate::Validate;
-use vipers::{assert_ata, assert_keys_eq};
 
 use crate::addresses;
 use crate::{
@@ -19,19 +20,12 @@ use crate::{
 
 impl<'info> Validate<'info> for NewRewarder<'info> {
     fn validate(&self) -> ProgramResult {
-        require!(self.base.is_signer, Unauthorized);
+        invariant!(self.base.is_signer, Unauthorized);
 
-        assert_ata!(
-            self.claim_fee_token_account,
-            self.rewarder,
-            self.rewards_token_mint
-        );
+        assert_keys_eq!(self.claim_fee_token_account.owner, self.rewarder);
+        assert_keys_eq!(self.claim_fee_token_account.mint, self.rewards_token_mint);
 
-        assert_keys_eq!(
-            self.mint_wrapper.token_mint,
-            self.rewards_token_mint,
-            "rewards token mint"
-        );
+        assert_keys_eq!(self.mint_wrapper.token_mint, self.rewards_token_mint);
 
         Ok(())
     }
@@ -40,14 +34,14 @@ impl<'info> Validate<'info> for NewRewarder<'info> {
 impl<'info> Validate<'info> for SetPauseAuthority<'info> {
     fn validate(&self) -> ProgramResult {
         self.auth.validate()?;
-        require!(!self.auth.rewarder.is_paused, Paused);
+        invariant!(!self.auth.rewarder.is_paused, Paused);
         Ok(())
     }
 }
 
 impl<'info> Validate<'info> for MutableRewarderWithPauseAuthority<'info> {
     fn validate(&self) -> ProgramResult {
-        require!(self.pause_authority.is_signer, Unauthorized);
+        invariant!(self.pause_authority.is_signer, Unauthorized);
         assert_keys_eq!(
             self.rewarder.pause_authority,
             self.pause_authority,
@@ -59,8 +53,8 @@ impl<'info> Validate<'info> for MutableRewarderWithPauseAuthority<'info> {
 
 impl<'info> Validate<'info> for TransferAuthority<'info> {
     fn validate(&self) -> ProgramResult {
-        require!(!self.rewarder.is_paused, Paused);
-        require!(self.authority.is_signer, Unauthorized);
+        invariant!(!self.rewarder.is_paused, Paused);
+        invariant!(self.authority.is_signer, Unauthorized);
         assert_keys_eq!(self.authority, self.rewarder.authority);
         Ok(())
     }
@@ -68,8 +62,8 @@ impl<'info> Validate<'info> for TransferAuthority<'info> {
 
 impl<'info> Validate<'info> for AcceptAuthority<'info> {
     fn validate(&self) -> ProgramResult {
-        require!(!self.rewarder.is_paused, Paused);
-        require!(
+        invariant!(!self.rewarder.is_paused, Paused);
+        invariant!(
             self.rewarder.pending_authority != Pubkey::default(),
             PendingAuthorityNotSet
         );
@@ -85,7 +79,7 @@ impl<'info> Validate<'info> for AcceptAuthority<'info> {
 impl<'info> Validate<'info> for SetAnnualRewards<'info> {
     fn validate(&self) -> ProgramResult {
         self.auth.validate()?;
-        require!(!self.auth.rewarder.is_paused, Paused);
+        invariant!(!self.auth.rewarder.is_paused, Paused);
         Ok(())
     }
 }
@@ -97,7 +91,7 @@ impl<'info> Validate<'info> for SetAnnualRewards<'info> {
 impl<'info> Validate<'info> for CreateQuarry<'info> {
     fn validate(&self) -> ProgramResult {
         self.auth.validate()?;
-        require!(!self.auth.rewarder.is_paused, Paused);
+        invariant!(!self.auth.rewarder.is_paused, Paused);
         Ok(())
     }
 }
@@ -105,7 +99,7 @@ impl<'info> Validate<'info> for CreateQuarry<'info> {
 impl<'info> Validate<'info> for SetRewardsShare<'info> {
     fn validate(&self) -> ProgramResult {
         self.auth.validate()?;
-        require!(!self.auth.rewarder.is_paused, Paused);
+        invariant!(!self.auth.rewarder.is_paused, Paused);
         assert_keys_eq!(self.quarry.rewarder_key, self.auth.rewarder, "rewarder");
         Ok(())
     }
@@ -114,7 +108,7 @@ impl<'info> Validate<'info> for SetRewardsShare<'info> {
 impl<'info> Validate<'info> for SetFamine<'info> {
     fn validate(&self) -> ProgramResult {
         self.auth.validate()?;
-        require!(!self.auth.rewarder.is_paused, Paused);
+        invariant!(!self.auth.rewarder.is_paused, Paused);
         assert_keys_eq!(self.quarry.rewarder_key, self.auth.rewarder, "rewarder");
         Ok(())
     }
@@ -122,7 +116,7 @@ impl<'info> Validate<'info> for SetFamine<'info> {
 
 impl<'info> Validate<'info> for UpdateQuarryRewards<'info> {
     fn validate(&self) -> ProgramResult {
-        require!(!self.rewarder.is_paused, Paused);
+        invariant!(!self.rewarder.is_paused, Paused);
         assert_keys_eq!(self.quarry.rewarder_key, self.rewarder, "rewarder");
         Ok(())
     }
@@ -134,10 +128,9 @@ impl<'info> Validate<'info> for UpdateQuarryRewards<'info> {
 
 impl<'info> Validate<'info> for CreateMiner<'info> {
     fn validate(&self) -> ProgramResult {
-        require!(!self.rewarder.is_paused, Paused);
-        assert_ata!(self.miner_vault, self.miner, self.token_mint, "miner vault");
-        assert_keys_eq!(self.miner_vault.owner, self.miner, "miner vault owner");
-        assert_keys_eq!(self.miner_vault.mint, self.token_mint, "miner vault mint");
+        invariant!(!self.rewarder.is_paused, Paused);
+        assert_keys_eq!(self.miner_vault.owner, self.miner);
+        assert_keys_eq!(self.miner_vault.mint, self.token_mint);
         assert_keys_eq!(
             self.miner_vault.mint,
             self.quarry.token_mint_key,
@@ -153,7 +146,7 @@ impl<'info> Validate<'info> for ClaimRewards<'info> {
     /// Validates a [ClaimRewards] accounts struct.
     fn validate(&self) -> ProgramResult {
         self.stake.validate()?;
-        require!(!self.stake.rewarder.is_paused, Paused);
+        invariant!(!self.stake.rewarder.is_paused, Paused);
 
         assert_keys_eq!(
             self.mint_wrapper.token_mint,
@@ -187,9 +180,8 @@ impl<'info> Validate<'info> for ClaimRewards<'info> {
 
         // claim_fee_token_account validate
         assert_keys_eq!(
-            *self.claim_fee_token_account,
-            self.stake.rewarder.claim_fee_token_account,
-            "claim_fee_token_account"
+            self.claim_fee_token_account,
+            self.stake.rewarder.claim_fee_token_account
         );
         assert_keys_eq!(
             self.claim_fee_token_account.mint,
@@ -203,9 +195,9 @@ impl<'info> Validate<'info> for ClaimRewards<'info> {
 
 impl<'info> Validate<'info> for UserClaim<'info> {
     fn validate(&self) -> ProgramResult {
-        require!(!self.rewarder.is_paused, Paused);
+        invariant!(!self.rewarder.is_paused, Paused);
         // authority
-        require!(self.authority.is_signer, Unauthorized);
+        invariant!(self.authority.is_signer, Unauthorized);
         assert_keys_eq!(self.authority, self.miner.authority, "miner authority");
 
         // quarry
@@ -221,9 +213,9 @@ impl<'info> Validate<'info> for UserClaim<'info> {
 impl<'info> Validate<'info> for UserStake<'info> {
     /// Validates the UserStake.
     fn validate(&self) -> ProgramResult {
-        require!(!self.rewarder.is_paused, Paused);
+        invariant!(!self.rewarder.is_paused, Paused);
         // authority
-        require!(self.authority.is_signer, Unauthorized);
+        invariant!(self.authority.is_signer, Unauthorized);
         assert_keys_eq!(self.authority, self.miner.authority, "miner authority");
 
         // quarry
@@ -254,18 +246,16 @@ impl<'info> Validate<'info> for UserStake<'info> {
 
 impl<'info> Validate<'info> for ExtractFees<'info> {
     fn validate(&self) -> ProgramResult {
-        require!(!self.rewarder.is_paused, Paused);
-        assert_ata!(
+        invariant!(!self.rewarder.is_paused, Paused);
+        assert_keys_eq!(
             self.claim_fee_token_account,
-            self.rewarder,
+            self.rewarder.claim_fee_token_account
+        );
+        assert_keys_eq!(
+            self.claim_fee_token_account.mint,
             self.rewarder.rewards_token_mint
         );
 
-        assert_keys_eq!(
-            self.claim_fee_token_account.mint,
-            self.rewarder.rewards_token_mint,
-            "claim_fee_token_account.mint"
-        );
         assert_keys_eq!(
             self.fee_to_token_account.mint,
             self.rewarder.rewards_token_mint,
@@ -276,11 +266,10 @@ impl<'info> Validate<'info> for ExtractFees<'info> {
             addresses::FEE_TO,
             "fee_to_token_account.owner"
         );
-        assert_ata!(
-            self.fee_to_token_account,
-            addresses::FEE_TO,
-            self.rewarder.rewards_token_mint,
-            "fee ata"
+        assert_keys_eq!(self.fee_to_token_account.owner, addresses::FEE_TO);
+        assert_keys_eq!(
+            self.fee_to_token_account.mint,
+            self.rewarder.rewards_token_mint
         );
 
         Ok(())
@@ -289,7 +278,7 @@ impl<'info> Validate<'info> for ExtractFees<'info> {
 
 impl<'info> Validate<'info> for MutableRewarderWithAuthority<'info> {
     fn validate(&self) -> ProgramResult {
-        require!(self.authority.is_signer, Unauthorized);
+        invariant!(self.authority.is_signer, Unauthorized);
         assert_keys_eq!(self.rewarder.authority, self.authority, "authority");
         Ok(())
     }
@@ -298,7 +287,7 @@ impl<'info> Validate<'info> for MutableRewarderWithAuthority<'info> {
 impl<'info> Validate<'info> for ReadOnlyRewarderWithAuthority<'info> {
     /// Validates the [crate::Rewarder] is correct.
     fn validate(&self) -> ProgramResult {
-        require!(self.authority.is_signer, Unauthorized);
+        invariant!(self.authority.is_signer, Unauthorized);
         assert_keys_eq!(self.authority, self.rewarder.authority);
         Ok(())
     }
